using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Web.Services;
using System.Configuration;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Security;
using Microsoft.Web.Services3.Security.Tokens;
using Microsoft.Web.Services3.Security.X509;

using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.Staging;
using CMS.GlobalHelper;
using CMS.EventLog;
using CMS.LicenseProvider;
using CMS.FileManager;
using CMS.SettingsProvider;
using CMS.WorkflowEngine;
using CMS.PortalEngine;
using CMS.DataEngine;

[System.Web.Services.WebService(Namespace = "http://localhost/SyncWebService/SyncServer")]
[Policy("ServerPolicy")]
public class SyncServer : WebService
{
    private string mExcludeColumns = "NodeID;NodeSiteID;NodeParentID;NodeACLID;NodeSKUID;DocumentCheckedOutVersionHistoryID;DocumentPublishedVersionHistoryID;DocumentWorkflowStepID;DocumentCheckedOutByUserID;DocumentNodeID;DocumentID;DocumentForeignKeyValue";
    private TreeProvider mTreeProvider = null;
    private SiteInfo currentSite = null;
    private string errorMessage = null;
    private int nodeId = 0;
    private string documentName = null;

    /// <summary>
    /// TreeProvider object to use
    /// </summary>
    private TreeProvider TreeProvider
    {
        get
        {
            if (mTreeProvider == null)
            {
                mTreeProvider = new TreeProvider();
            }
            return mTreeProvider;
        }
        set
        {
            mTreeProvider = value;
        }
    }


    /// <summary>
    /// Returns the full error message
    /// </summary>
    private string GetError(string errorMessage)
    {
        return ResHelper.GetString("SyncServer.ServerError") + ": " + errorMessage;
    }


    public SyncServer()
    {
    }


    /// <summary>
    /// Processes the given synchronization task
    /// </summary>
    /// <param name="taskTitle">Title of the synchronization task</param>
    /// <param name="taskType">Type of the synchronization task</param>
    /// <param name="taskData">Document data for the synchronization task</param>
    [WebMethod()]
    public string ProcessSyncTask(string taskTitle, string taskType, string taskData)
    {
        // Get current site
        currentSite = CMSContext.CurrentSite;
        if (currentSite == null)
        {
            return GetError("[SyncServer.ProcessSyncTask]: Site not running.");
        }

        // Check license
        if (!LicenseHelper.CheckFeature(HTTPHelper.GetAbsoluteUri(), FeatureEnum.Staging))
        {
            return GetError(ResHelper.GetString("SyncServer.ErrorLicense"));
        }

        try
        {
            // If not enabled, refuse to serve
            if (!StagingHelper.IsServerEnabled(currentSite.SiteName))
            {
                return GetError(ResHelper.GetString("SyncServer.ErrorServiceNotEnabled"));
            }

            // Reject any requests which are not valid SOAP requests
            SoapContext currentContext = RequestSoapContext.Current;
            if (currentContext == null)
            {
                return "[SyncServer.ProcessSyncTask]: Only SOAP requests are permitted.";
            }

            // Check the authentication
            switch (StagingHelper.ServerAuthenticationType(currentSite.SiteName))
            {
                // Username and password authentication
                case ServerAuthenticationEnum.UserName:
                    break;

                // X509 authentication
                case ServerAuthenticationEnum.X509:
                    // Verify message content
                    if (!VerifyMessageParts(currentContext))
                    {
                        return GetError(errorMessage);
                    }
                    break;

                default:
                    return GetError(ResHelper.GetString("SyncServer.ErrorInvalidAuthentication"));
            }

            // Check task type presence
            if (taskType == null)
            {
                return "[SyncServer.ProcessSyncTask]: Missing task type.";
            }
            // Get the document data
            if ((taskData == null) || (taskData == ""))
            {
                return "[SyncServer.ProcessSyncTask]: Missing task data.";
            }
            DataSet taskDS = GetDataSet(taskData);
            if (taskDS == null)
            {
                return errorMessage;
            }

            // Process the task
            switch (taskType.ToLower())
            {
                // Update document
                case "updatedoc":
                    UpdateDocument(taskDS);
                    break;

                // Document deletion
                case "deletedoc":
                    DeleteDocument(taskDS, false);
                    break;

                // Delete all culture versions of the document
                case "deleteallcultures":
                    DeleteDocument(taskDS, true);
                    break;

                // Publish document
                case "publishdoc":
                    PublishDocument(taskDS);
                    break;

                // Move document
                case "movedoc":
                    MoveDocument(taskDS);
                    break;

                // Unknown task
                default:
                    return GetError(ResHelper.GetString("SyncServer.ErrorUnknownTask"));
            }

            return errorMessage;
        }
        catch (Exception ex)
        {
            // Log the exception
            EventLogProvider ev = new EventLogProvider();
            ev.LogEvent("E", DateTime.Now, "Staging", "RUNTASK", 0, null, nodeId, documentName, HTTPHelper.GetUserHostAddress(), ex.Message + ": " + ex.StackTrace, currentSite.SiteID, HTTPHelper.GetAbsoluteUri());

            // Return the error message
            return GetError(ResHelper.GetString("SyncServer.ErrorException") + ": " + ex.Message);
        }
    }


    /// <summary>
    /// Updates the document
    /// </summary>
    /// <param name="documentDS">Document data</param>
    private void UpdateDocument(DataSet documentDS)
    {
        // Get the document table
        DataTable documentTable = documentDS.Tables["DocumentData"];
        if (documentTable == null)
        {
            errorMessage = "[SyncServer.UpdateDocument]: Document data table not found.";
            return;
        }

        // Get the translation table
        DataTable transTable = documentDS.Tables["ObjectTranslation"];
        if (transTable == null)
        {
            errorMessage = "[SyncServer.UpdateDocument]: Object translation table not found.";
            return;
        }
        TranslationHelper th = new TranslationHelper(transTable);

        // Get the atachments table
        DataTable attachmentsTable = documentDS.Tables["CMS_Attachment"];
        if (attachmentsTable != null)
        {
            // Translate the binaries
            DataHelper.TranslateBinaryColumn(attachmentsTable, "AttachmentBinary");
        }
        AttachmentManager am = new AttachmentManager(this.TreeProvider.Connection);

        // Get relationships table
        DataTable relationshipsTable = documentDS.Tables["CMS_Relationship"];

        // Update the documents within table
        foreach (DataRow nodeRow in documentTable.Rows)
        {
            // Get the document class
            int classId = ValidationHelper.GetInteger(DataHelper.GetDataRowValue(nodeRow, "NodeClassID"), 0);
            string className = th.GetCodeName("CMS.Class", classId);
            DataClassInfo ci = DataClassInfoProvider.GetDataClass(className);
            if (ci == null)
            {
                // Class not found
                errorMessage = "[SyncServer.UpdateDocument]: Class name '" + className + "' not found.";
                return;
            }

            // Create the document node
            CMS.TreeEngine.TreeNode node = new TreeNode(nodeRow, className, this.TreeProvider);
            int documentId = node.DocumentID;

            // Remove excluded column values
            RemoveExcludedValues(node);

            // Translate dependent columns
            TranslateNodeColumns(node, th);

            // Get current node by GUID
            CMS.TreeEngine.TreeNode currentNode = this.TreeProvider.SelectSingleNode(node.NodeGUID, TreeProvider.ALL_CULTURES, currentSite.SiteName);
            // Node not exists, insert new node
            if (currentNode == null)
            {
                // Get node parent
                string parentAliasPath = TreePathUtils.GetParentPath(node.NodeAliasPath);
                CMS.TreeEngine.TreeNode parentNode = this.TreeProvider.SelectSingleNode(currentSite.SiteName, parentAliasPath, TreeProvider.ALL_CULTURES, false, null, false);
                if (parentNode == null)
                {
                    // Parent not found
                    errorMessage = GetError(ResHelper.GetString("SyncServer.ErrorParentNotFound"));
                    return;
                }

                // Insert the document
                node.Insert(parentNode.NodeID);
                //DocumentHelper.InsertDocument(node, parentNode.NodeID, this.TreeProvider);
            }
            else
            {
                // Get current document culture
                CMS.TreeEngine.TreeNode cultureNode = this.TreeProvider.SelectSingleNode(currentSite.SiteName, currentNode.NodeAliasPath, node.DocumentCulture, false, className, false);
                if (cultureNode != null)
                {
                    // Document culture exists, update
                    DocumentHelper.CopyNodeData(node, cultureNode, true, true, false, true, true, mExcludeColumns);
                    DocumentHelper.CopyNodeData(node, cultureNode, false, false, true, true, false, mExcludeColumns);

                    cultureNode.Update();
                    //DocumentHelper.UpdateDocument(cultureNode, this.TreeProvider);
                    node = cultureNode;
                }
                else
                {
                    // Document culture not exists, insert new culture version
                    DocumentHelper.CopyNodeData(node, currentNode, true, true, true, true, true, mExcludeColumns);

                    currentNode.InsertAsNewCultureVersion();
                    //DocumentHelper.InsertNewCultureVersion(currentNode, this.TreeProvider);                
                    node = currentNode;
                }
            }

            // Update attachments
            if (attachmentsTable != null)
            {
                // Get current document attachments
                DataSet currentAttachmentsDS = am.GetAttachments(node.DocumentID);
                DataTable currentAttachmentsTable = null;
                if (!DataHelper.DataSourceIsEmpty(currentAttachmentsDS))
                {
                    currentAttachmentsTable = currentAttachmentsDS.Tables[0];
                }

                DataRow[] attachments = attachmentsTable.Select("AttachmentDocumentID = " + documentId.ToString());
                if ((attachments != null) && (attachments.Length > 0))
                {
                    // Process all the atachments
                    foreach (DataRow attachmentRow in attachments)
                    {
                        // Get current attachment
                        Guid attachmentGuid = ValidationHelper.GetGuid(attachmentRow["AttachmentGUID"], Guid.Empty);
                        if (attachmentGuid != Guid.Empty)
                        {
                            // Get current attachment
                            DataRow currentAttachment = null;
                            if (currentAttachmentsTable != null)
                            {
                                DataRow[] currentAttachments = currentAttachmentsTable.Select("AttachmentGUID = '" + attachmentGuid.ToString() + "'");
                                if ((currentAttachments != null) && (currentAttachments.Length > 0))
                                {
                                    currentAttachment = currentAttachments[0];
                                }
                            }

                            // Create new attachment
                            AttachmentInfo newAttachment = new AttachmentInfo(attachmentRow, this.TreeProvider.Connection);
                            if (currentAttachment != null)
                            {
                                newAttachment.AttachmentID = ValidationHelper.GetInteger(currentAttachment["AttachmentID"], 0);
                                // Remove current attachment row to mark as processed
                                currentAttachmentsTable.Rows.Remove(currentAttachment);
                            }
                            else
                            {
                                newAttachment.AttachmentID = 0;
                            }
                            newAttachment.AttachmentDocumentID = node.DocumentID;
                            // Save attachment
                            am.SetAttachmentInfo(newAttachment);
                        }
                    }
                }

                // Delete all leaving current attachments (those that are over new document data)
                if (currentAttachmentsTable != null)
                {
                    foreach (DataRow dr in currentAttachmentsTable.Rows)
                    {
                        int attachmentId = ValidationHelper.GetInteger(dr["AttachmentID"], 0);
                        if (attachmentId > 0)
                        {
                            am.DeleteAttachmentInfo(attachmentId);
                        }
                    }
                }
            }
            else
            {
                am.DeleteAtachments(node.DocumentID);
            }

            // Update relationships
            UpdateRelationships(node, relationshipsTable);
        }
    }


    /// <summary>
    /// Publishes the document
    /// </summary>
    /// <param name="documentDS">Document data</param>
    private void PublishDocument(DataSet documentDS)
    {
        // Get the document table
        DataTable documentTable = documentDS.Tables["DocumentData"];
        if (documentTable == null)
        {
            errorMessage = "[SyncServer.PublishDocument]: Document data table not found.";
            return;
        }

        // Get Version history data table
        DataTable versionHistoryTable = documentDS.Tables["CMS_VersionHistory"];

        // Get Attachment history table
        DataTable attachmentHistoryTable = documentDS.Tables["CMS_AttachmentHistory"];
        // Translate the binary column
        if (attachmentHistoryTable != null)
        {
            DataHelper.TranslateBinaryColumn(attachmentHistoryTable, "AttachmentBinary");
        }

        // Get relationships table
        DataTable relationshipsTable = documentDS.Tables["CMS_Relationship"];

        // Get the translation table
        DataTable transTable = documentDS.Tables["ObjectTranslation"];
        if (transTable == null)
        {
            errorMessage = "[SyncServer.PublishDocument]: Object translation table not found.";
            return;
        }
        TranslationHelper th = new TranslationHelper(transTable);

        VersionManager vm = new VersionManager(this.TreeProvider);
        WorkflowManager wm = new WorkflowManager(this.TreeProvider);

        // Publish the document versions
        foreach (DataRow nodeRow in documentTable.Rows)
        {
            // Get node GUID
            Guid nodeGuid = ValidationHelper.GetGuid(DataHelper.GetDataRowValue(nodeRow, "NodeGUID"), Guid.Empty);
            if (nodeGuid == Guid.Empty)
            {
                errorMessage = "[SyncServer.PublishDocument]: Node GUID not specified.";
                return;
            }

            // Get document culture
            string culture = ValidationHelper.GetString(DataHelper.GetDataRowValue(nodeRow, "DocumentCulture"), "");
            if (culture == "")
            {
                errorMessage = "[SyncServer.PublishDocument]: Document culture not specified.";
                return;
            }

            // Get current node by GUID
            CMS.TreeEngine.TreeNode currentNode = this.TreeProvider.SelectSingleNode(nodeGuid, culture, currentSite.SiteName);
            if (currentNode == null)
            {
                errorMessage = GetError(ResHelper.GetString("SyncServer.ErrorNodeNotFound"));
                return;
            }

            // Create the document node
            CMS.TreeEngine.TreeNode node = new TreeNode(nodeRow, currentNode.NodeClassName, this.TreeProvider);
            int documentId = node.DocumentID;

            
            // Remove excluded column values
            RemoveExcludedValues(node);

            // Translate dependent columns
            TranslateNodeColumns(node, th);


            // Get the version data
            if (versionHistoryTable != null)
            {
                DataRow[] versions = versionHistoryTable.Select("DocumentID = " + documentId.ToString());
                if ( (versions != null) && (versions.Length>0) )
                {
                    // Save all the versions
                    foreach (DataRow versionRow in versions)
                    {
                        // Update current document with not versioned columns
                        DocumentHelper.CopyNodeData(node, currentNode, true, true, false, false, true, mExcludeColumns);
                        // Update UrlPath if generated
                        if (currentNode.DocumentUseNamePathForUrlPath)
                        {
                            currentNode.SetValue("DocumentUrlPath", TreePathUtils.GetUrlPathFromNamePath((string)node.DocumentNamePath));
                        }
                        currentNode.NodeName = node.NodeName;
                        currentNode.Update();

                        // Clear the document version history queue
                        vm.DeleteOlderVersions(currentNode.DocumentID, currentSite.SiteName);

                        // Insert the version object
                        DataClass versionObj = new DataClass("CMS.VersionHistory", versionRow, this.TreeProvider.Connection);
                        int versionHistoryId = ValidationHelper.GetInteger(versionObj.GetValue("VersionHistoryID"), 0);
                        // Update fields
                        versionObj.SetValue("DocumentID", currentNode.DocumentID);
                        versionObj.SetValue("NodeSiteID", currentNode.NodeSiteID);
                        versionObj.SetValue("ModifiedByUserID", node.GetValue("DocumentModifiedByUserID"));
                        versionObj.SetValue("ToBePublished", true);
                        versionObj.SetValue("WasPublishedFrom", null);
                        versionObj.SetValue("WasPublishedTo", null);
                        // Set data
                        versionObj.SetValue("NodeXML", node.DataRow.Table.DataSet.GetXml());

                        versionObj.Insert();
                        int newVersionHistoryId = ValidationHelper.GetInteger(versionObj.GetValue("VersionHistoryID"), 0);

                        // Update the node data
                        currentNode.SetValue("DocumentCheckedOutVersionHistoryID", newVersionHistoryId);
                        currentNode.SetValue("DocumentCheckedOutByUserID", null);
                        // Set current step to published
                        WorkflowInfo wi = wm.GetNodeWorkflow(currentNode);
                        if (wi != null)
                        {
                            WorkflowStepInfo si = WorkflowInfoProvider.GetPublishedStep(wi.WorkflowID);
                            if (si != null)
                            {
                                currentNode.SetValue("DocumentWorkflowStepID", si.StepID);
                            }
                        }
                        currentNode.Update();

                        // Get the version attachments
                        if (attachmentHistoryTable != null)
                        {
                            DataRow[] attachments = attachmentHistoryTable.Select("VersionHistoryID = " + versionHistoryId.ToString());
                            if ((attachments != null) && (attachments.Length > 0))
                            {
                                // Process all the attachments
                                foreach (DataRow attachmentRow in attachments)
                                {
                                    AttachmentInfo ai = new AttachmentInfo(attachmentRow, this.TreeProvider.Connection);
                                    vm.SaveAttachmentVersion(ai, newVersionHistoryId);
                                }
                            }
                        }
                    }
                }
            }

            // Update relationships
            UpdateRelationships(currentNode, relationshipsTable);
        }
    }


    /// <summary>
    /// Deletes the document
    /// </summary>
    /// <param name="documentDS">Document data</param>
    /// <param name="deleteAllCultures">Delete all culture versions</param>
    private void DeleteDocument(DataSet documentDS, bool deleteAllCultures)
    {
        // Get the document table
        DataTable documentTable = documentDS.Tables["DocumentData"];
        if (documentTable == null)
        {
            errorMessage = "[SyncServer.DeleteDocument]: Document data table not found.";
            return;
        }

        // Delete the documents within table
        foreach (DataRow nodeRow in documentTable.Rows)
        {
            // Get node GUID
            Guid nodeGuid = ValidationHelper.GetGuid(DataHelper.GetDataRowValue(nodeRow, "NodeGUID"), Guid.Empty);
            if (nodeGuid == Guid.Empty)
            {
                errorMessage = "[SyncServer.DeleteDocument]: Node GUID not specified.";
                return;
            }

            // Get document culture
            string culture = ValidationHelper.GetString(DataHelper.GetDataRowValue(nodeRow, "DocumentCulture"), "");
            if (culture == "")
            {
                errorMessage = "[SyncServer.DeleteDocument]: Document culture not specified.";
                return;
            }

            // Get current node by GUID
            CMS.TreeEngine.TreeNode currentNode = this.TreeProvider.SelectSingleNode(nodeGuid, TreeProvider.ALL_CULTURES, currentSite.SiteName);
            if (currentNode != null)
            {
                // Delete all cultures?
                if (deleteAllCultures)
                {
                    DocumentHelper.DeleteDocument(currentNode, this.TreeProvider, true, true);
                }
                else
                {
                    // Get current culture node
                    CMS.TreeEngine.TreeNode cultureNode = this.TreeProvider.SelectSingleNode(currentSite.SiteName, currentNode.NodeAliasPath, culture, false, currentNode.NodeClassName, false);
                    if (culture != null)
                    {
                        // Delete document
                        DocumentHelper.DeleteDocument(cultureNode, this.TreeProvider, false, true);
                    }
                }
            }
        }

    }


    /// <summary>
    /// Moves the document
    /// </summary>
    /// <param name="documentDS">Document data</param>
    private void MoveDocument(DataSet documentDS)
    {
        // Get the document table
        DataTable documentTable = documentDS.Tables["DocumentData"];
        if (documentTable == null)
        {
            errorMessage = "[SyncServer.DeleteDocument]: Document data table not found.";
            return;
        }

        // Move the documents within table
        foreach (DataRow nodeRow in documentTable.Rows)
        {
            // Get node GUID
            Guid nodeGuid = ValidationHelper.GetGuid(DataHelper.GetDataRowValue(nodeRow, "NodeGUID"), Guid.Empty);
            if (nodeGuid == Guid.Empty)
            {
                errorMessage = "[SyncServer.DeleteDocument]: Node GUID not specified.";
                return;
            }

            // Get document culture
            string culture = ValidationHelper.GetString(DataHelper.GetDataRowValue(nodeRow, "DocumentCulture"), "");
            if (culture == "")
            {
                errorMessage = "[SyncServer.DeleteDocument]: Document culture not specified.";
                return;
            }

            // Get current node by GUID
            CMS.TreeEngine.TreeNode currentNode = this.TreeProvider.SelectSingleNode(nodeGuid, TreeProvider.ALL_CULTURES, currentSite.SiteName);
            if (currentNode != null)
            {
                // Get target parent node
                string aliasPath = ValidationHelper.GetString(DataHelper.GetDataRowValue(nodeRow, "NodeAliasPath"), "");
                string parentAliasPath = TreePathUtils.GetParentPath(aliasPath);
                CMS.TreeEngine.TreeNode parentNode = this.TreeProvider.SelectSingleNode(currentSite.SiteName, parentAliasPath, TreeProvider.ALL_CULTURES, false, null, false);
                if (parentNode == null)
                {
                    // Parent not found
                    errorMessage = GetError(ResHelper.GetString("SyncServer.ErrorParentNotFound"));
                    return;
                }

                // Move the document
                DocumentHelper.MoveDocument(currentNode, parentNode.NodeID, this.TreeProvider);
            }
            else
            {
                errorMessage = GetError(ResHelper.GetString("SyncServer.ErrorDocumentNotFound"));
            }
        }

    }


    /// <summary>
    /// Vefies, if all the required message parts are present
    /// </summary>
    /// <param name="context">Soap context</param>
    private bool VerifyMessageParts(SoapContext context)
    {
        // Body
        if (context.Envelope.Body == null)
        {
            errorMessage = "The message must contain a soap:Body element";
            return false;
        }
        // To
        if ((context.Addressing.To == null) || (context.Addressing.To.TargetElement == null))
        {
            errorMessage = "The message must contain a wsa:To header";
            return false;
        }
        // Action
        if ((context.Addressing.Action == null) || (context.Addressing.Action.TargetElement == null))
        {
            errorMessage = "The message must contain a wsa:Action header";
            return false;
        }
        // Message ID
        if (context.Addressing.MessageID == null || context.Addressing.MessageID.TargetElement == null)
        {
            errorMessage = "The message must contain a wsa:MessageID header";
            return false;
        }

        // Message OK
        return true;
    }


    /// <summary>
    /// Returns the dataset loaded from the given document data
    /// </summary>
    /// <param name="documentData">Document data to make the dataset from</param>
    private DataSet GetDataSet(string documentData)
    {
        DataSet ds = null;
        try
        {
            XmlParserContext xmlContext = new XmlParserContext(null, null, null, XmlSpace.None);
            XmlReader reader = new XmlTextReader(documentData, XmlNodeType.Element, xmlContext);
            ds = new DataSet();
            ds.ReadXml(reader);
        }
        catch (Exception ex)
        {
            errorMessage = "[SyncServer.GetDataSet]: Error parsing document data - " + ex.Message;
            return null;
        }
        return ds;
    }


    /// <summary>
    /// Updates the relationships
    /// </summary>
    /// <param name="node">Document node</param>
    /// <param name="relationshipsTable">Relationships table</param>
    private void UpdateRelationships(CMS.TreeEngine.TreeNode node, DataTable relationshipsTable)
    {
        // Update relationships
        if (relationshipsTable != null)
        {
            // Get current relationships
            DataSet currentRelationshipsDS = RelationshipProvider.GetRelationships(node.NodeID, true, true);
            DataTable currentRelationshipsTable = null;
            if (!DataHelper.DataSourceIsEmpty(currentRelationshipsDS))
            {
                currentRelationshipsTable = currentRelationshipsDS.Tables[0];
            }

            // Process all the relationships
            foreach (DataRow relationshipRow in relationshipsTable.Rows)
            {
                Guid leftNodeGUID = ValidationHelper.GetGuid(relationshipRow["LeftNodeGUID"], Guid.Empty);
                Guid rightNodeGUID = ValidationHelper.GetGuid(relationshipRow["RightNodeGUID"], Guid.Empty);
                string relationshipName = ValidationHelper.GetString(relationshipRow["RelationshipName"], "");

                // Get current relationship
                DataRow currentRelationship = null;
                if (currentRelationshipsTable != null)
                {
                    DataRow[] currentRelationships = currentRelationshipsTable.Select("LeftNodeGUID = '" + leftNodeGUID.ToString() + "' AND RightNodeGUID = '" + rightNodeGUID + "' AND RelationshipName = '" + relationshipName + "'");
                    if ((currentRelationships != null) && (currentRelationships.Length > 0))
                    {
                        currentRelationship = currentRelationships[0];
                    }
                }
                if (currentRelationship != null)
                {
                    // Relationship exists, remove from the table
                    currentRelationshipsTable.Rows.Remove(currentRelationship);
                }
                else
                {
                    int leftNodeId = TreePathUtils.GetNodeIdByNodeGUID(leftNodeGUID, currentSite.SiteName);
                    int rightNodeId = TreePathUtils.GetNodeIdByNodeGUID(rightNodeGUID, currentSite.SiteName);

                    // Not exists, add relationship if relationshipname exists
                    RelationshipNameInfo ri = RelationshipNameInfoProvider.GetRelationshipNameInfo(relationshipName);
                    if ((leftNodeId > 0) && (rightNodeId > 0) && (ri != null) && ri.IsInSite(currentSite.SiteID))
                    {
                        RelationshipProvider.AddRelationship(leftNodeId, rightNodeId, ri.RelationshipNameId);
                    }
                }
            }

            // Delete all remaining relationships
            if (currentRelationshipsTable != null)
            {
                foreach (DataRow dr in currentRelationshipsTable.Rows)
                {
                    // Remove the relationship
                    int leftNodeId = ValidationHelper.GetInteger(dr["LeftNodeID"], 0);
                    int rightNodeId = ValidationHelper.GetInteger(dr["RightNodeID"], 0);
                    int relationshipNameId = ValidationHelper.GetInteger(dr["RelationshipNameID"], 0);
                    if ((leftNodeId > 0) && (rightNodeId > 0) && (relationshipNameId > 0))
                    {
                        RelationshipProvider.RemoveRelationship(leftNodeId, rightNodeId, relationshipNameId);
                    }
                }
            }
        }
        else
        {
            RelationshipProvider.DeleteRelationships(node.NodeID, true, true);
        }
    }


    /// <summary>
    /// Translates given node columns using data from helper
    /// </summary>
    /// <param name="node">Node to translate</param>
    /// <param name="th">Translation helper with the data</param>
    private void TranslateNodeColumns(CMS.TreeEngine.TreeNode node, TranslationHelper th)
    {
        // Translate page template ID to the new value
        if (node.DocumentPageTemplateID > 0)
        {
            string templateName = th.GetCodeName("CMS_PageTemplate", node.DocumentPageTemplateID);
            if (templateName != "")
            {
                PageTemplateInfo ti = PageTemplateInfoProvider.GetPageTemplateInfo(templateName);
                if (ti != null)
                {
                    node.DocumentPageTemplateID = ti.PageTemplateId;
                }
                else
                {
                    node.DocumentPageTemplateID = 0;
                }
            }
            else
            {
                node.DocumentPageTemplateID = 0;
            }
        }

        // Translate CSS stylesheet to the new value
        if (node.DocumentStylesheetID > 0)
        {
            string stylesheetName = th.GetCodeName("CMS_CssStylesheet", node.DocumentStylesheetID);
            if (stylesheetName != "")
            {
                CssStylesheetInfo si = CssStylesheetInfoProvider.GetCssStylesheetInfo(stylesheetName);
                if (si != null)
                {
                    node.DocumentStylesheetID = si.StylesheetID;
                }
                else
                {
                    node.DocumentStylesheetID = 0;
                }
            }
            else
            {
                node.DocumentStylesheetID = 0;
            }
        }

        // Translate user IDs to new values
        int createdByUserId = ValidationHelper.GetInteger(node.GetValue("DocumentCreatedByUserID"), 0);
        if (createdByUserId > 0)
        {
            string username = th.GetCodeName("CMS_User", createdByUserId, "administrator");
            if (username != "")
            {
                UserInfo ui = UserInfoProvider.GetUserInfo(username);
                if (ui != null)
                {
                    node.SetValue("DocumentCreatedByUserID", ui.UserID);
                }
                else
                {
                    node.SetValue("DocumentCreatedByUserID", null);
                }
            }
            else
            {
                node.SetValue("DocumentCreatedByUserID", null);
            }
        }

        int modifiedByUserId = ValidationHelper.GetInteger(node.GetValue("DocumentModifiedByUserID"), 0);
        if (modifiedByUserId > 0)
        {
            string username = th.GetCodeName("CMS_User", modifiedByUserId, "administrator");
            if (username != "")
            {
                UserInfo ui = UserInfoProvider.GetUserInfo(username);
                if (ui != null)
                {
                    node.SetValue("DocumentModifiedByUserID", ui.UserID);
                }
                else
                {
                    node.SetValue("DocumentModifiedByUserID", null);
                }
            }
            else
            {
                node.SetValue("DocumentModifiedByUserID", null);
            }
        }
    }


    /// <summary>
    /// Removes excluded column values from the given document
    /// </summary>
    private void RemoveExcludedValues(CMS.TreeEngine.TreeNode node)
    {
        string[] excluded = mExcludeColumns.Split(';');
        foreach (string col in excluded)
        {
            node.SetValue(col, null);
        }
    }

}
