using System;
using System.Collections;
using System.Data;
using System.IO;
using System.Security.Permissions;
using System.Web;
using System.Web.Caching;
using System.Web.Hosting;

using CMS.SettingsProvider;
using CMS.PortalEngine;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.GlobalHelper;

/// <summary>
/// Virtual path provider dynamicly loads content from virtual paths
/// </summary>
public class DbPathProvider : VirtualPathProvider
{
    /// <summary>
    /// Web parts virtual directory
    /// </summary>
    public const string VD_WEBPARTS = "CMSVirtualWebParts";

    /// <summary>
    /// Transformations virtual directory
    /// </summary>
    public const string VD_TRANSFORMATIONS = "CMSTransformations";

    /// <summary>
    /// Layouts virtual directory
    /// </summary>
    public const string VD_LAYOUTS = "CMSLayouts";

    /// <summary>
    /// Page layouts virtual directory
    /// </summary>
    public const string VD_PAGELAYOUTS = "CMSTemplateLayouts";

    /// <summary>
    /// CSS stylesheets virtual directory
    /// </summary>
    public const string VD_CSSSTYLESHEETS = "CMSCSSStylesheets";


    protected string mVirtualDirectory = null;
    private string mVirtualPath = null;
    private string mObjectName = null;
    private string mParameters = null;

    private TransformationInfo mTransformation = null;
    private LayoutInfo mLayout = null;
    private PageTemplateInfo mPageLayout = null;
    private CssStylesheetInfo mStylesheet = null;
    private WebPartInstance mWebPart = null;


    /// <summary>
    /// Processed transformation
    /// </summary>
    public TransformationInfo Transformation
    {
        get
        {
            if ((mTransformation == null) && (this.mVirtualDirectory == VD_TRANSFORMATIONS) && (this.mVirtualPath != null))
            {
                string virtualPath = mVirtualPath;

                // Process the parameters
                int paramIndex = virtualPath.IndexOf("---");
                if (paramIndex >= 0)
                {
                    mParameters = virtualPath.Substring(paramIndex);
                    virtualPath = virtualPath.Substring(0, paramIndex);
                }

                mObjectName = TransformationInfoProvider.GetTransformationName(virtualPath).ToLower();
                if (mObjectName != null)
                {
                    mTransformation = TransformationInfoProvider.GetTransformation(mObjectName);
                }
            }

            return mTransformation;
        }
    }


    /// <summary>
    /// Processed layout
    /// </summary>
    public LayoutInfo Layout
    {
        get
        {
            if ((mLayout == null) && (this.mVirtualDirectory == VD_LAYOUTS) && (this.mVirtualPath != null))
            {
                string virtualPath = mVirtualPath;

                // Process the parameters
                int paramIndex = virtualPath.IndexOf("---");
                if (paramIndex >= 0)
                {
                    mParameters = virtualPath.Substring(paramIndex);
                    virtualPath = virtualPath.Substring(0, paramIndex);
                }

                // Get the layout
                mObjectName =LayoutInfoProvider.GetLayoutName(virtualPath).ToLower();
                if (mObjectName != null)
                {
                    mLayout = LayoutInfoProvider.GetLayoutInfo(mObjectName);
                }
            }

            return mLayout;
        }
    }

    /// <summary>
    /// Processed layout
    /// </summary>
    public PageTemplateInfo PageLayout
    {
        get
        {
            if ((mPageLayout == null) && (this.mVirtualDirectory == VD_PAGELAYOUTS) && (this.mVirtualPath != null))
            {
                string virtualPath = mVirtualPath;

                // Process the parameters
                int paramIndex = virtualPath.IndexOf("---");
                if (paramIndex >= 0)
                {
                    mParameters = virtualPath.Substring(paramIndex);
                    virtualPath = virtualPath.Substring(0, paramIndex);
                }

                // Get the layout
                mObjectName = PageTemplateInfoProvider.GetPageLayoutName(virtualPath).ToLower();
                if (mObjectName != null)
                {
                    mPageLayout = PageTemplateInfoProvider.GetPageTemplateInfo(mObjectName);
                }
            }

            return mPageLayout;
        }
    }


    /// <summary>
    /// Processed stylesheet
    /// </summary>
    public CssStylesheetInfo Stylesheet
    {
        get
        {
            if ((mStylesheet == null) && (this.mVirtualDirectory == VD_CSSSTYLESHEETS) && (this.mVirtualPath != null))
            {
                string virtualPath = mVirtualPath;

                // Process the parameters
                int paramIndex = virtualPath.IndexOf("---");
                if (paramIndex >= 0)
                {
                    mParameters = virtualPath.Substring(paramIndex);
                    virtualPath = virtualPath.Substring(0, paramIndex);
                }

                // Get the stylesheet
                mObjectName = CssStylesheetInfoProvider.GetCSSStylesheetName(virtualPath).ToLower();
                if (mObjectName != null)
                {
                    mStylesheet = CssStylesheetInfoProvider.GetCssStylesheetInfo(mObjectName);
                }
            }

            return mStylesheet;
        }
    }


    /// <summary>
    /// Processed web part
    /// </summary>
    public WebPartInstance WebPart
    {
        get
        {
            if ( /*(mWebPart == null) &&*/ (this.mVirtualDirectory == VD_WEBPARTS) && (this.mVirtualPath != null))
            {
                string virtualPath = mVirtualPath;

                // Process the parameters
                int paramIndex = virtualPath.IndexOf("---");
                if (paramIndex >= 0)
                {
                    mParameters = virtualPath.Substring(paramIndex);
                    virtualPath = virtualPath.Substring(0, paramIndex);
                }

                string vPath = DbVirtualFile.GetWebPartParameters(virtualPath);
                mObjectName = vPath.ToLower();
                int lastSlash = vPath.LastIndexOf("/");
                if (lastSlash >= 0)
                {
                    // Web part ID
                    string webPartId = vPath.Substring(lastSlash + 1);
                    vPath = vPath.Substring(0, lastSlash);
                    // Zone ID
                    lastSlash = vPath.LastIndexOf("/");
                    string zoneId = vPath.Substring(lastSlash + 1);
                    string aliasPath = "/";
                    if (lastSlash >= 0)
                    {
                        aliasPath += vPath.Substring(0, lastSlash);
                    }

                    // Get the web part data
                    if ((webPartId != "") && (zoneId != "") && (aliasPath != ""))
                    {
                        // Get page info
                        PageInfo pi = PageInfoProvider.GetPageInfo(CMSContext.CurrentSite.SiteName, aliasPath, CMSContext.CurrentUser.PreferredCultureCode, null, CMSContext.CurrentSite.CombineWithDefaultCulture);
                        if (pi == null)
                        {
                            throw new Exception("[DbPathProvider.GetFileContent]: Page '" + aliasPath + "' not found.");
                        }

                        // If inherited, get parent page with template
                        if (pi.PageTemplateInfo.PageTemplateId <= 0)
                        {
                            pi = PageInfoProvider.GetParentPageInfo(pi);
                        }

                        // If template not found
                        if (pi.PageTemplateInfo.PageTemplateId <= 0)
                        {
                            throw new Exception("[DbPathProvider.GetFileContent]: Page template for page '" + aliasPath + "' not found.");
                        }

                        // Get zone instance
                        WebPartZoneInstance zone = pi.PageTemplateInfo.GetZone(zoneId);
                        if (zone == null)
                        {
                            throw new Exception("[DbPathProvider.GetFileContent]: Zone '" + zoneId + "' not found.");
                        }

                        // Get webpart instance
                        mWebPart = zone.GetWebPart(webPartId);
                        if ( (mWebPart == null) && (webPartId.ToLower().EndsWith(".ascx.cs")) || (webPartId.ToLower().EndsWith(".ascx.vb")) )
                        {
                            throw new Exception("[DbPathProvider.GetFileContent]: Web part '" + this.mVirtualPath + "' does not support virtual path provider access, please make sure that the CodeFile attribute contains the full file system path (e.g. CodeFile=\"~/CMSWebParts/Category/CodeName.ascx.cs\").");
                        }
                    }
                }

            }

            return mWebPart;
        }
    }


    /// <summary>
    /// Public property 'VirtualDirectory'.
    /// </summary>
    public string VirtualDirectory
    {
        get
        {
            if ((mVirtualDirectory == null) && (mVirtualPath != null))
            {
                mVirtualDirectory = GetVirtDirectory(mVirtualPath);
            }
            return mVirtualDirectory;
        }
    }


    /// <summary>
    /// Current virtual path
    /// </summary>
    public string VirtualPath
    {
        get
        {
            return mVirtualPath;
        }
        set
        {
            if (mVirtualPath != value)
            {
                mVirtualPath = value;
                mVirtualDirectory = null;
                mTransformation = null;
                mLayout = null;
                mStylesheet = null;
                mPageLayout = null;
            }
        }
    }


    /// <summary>
    /// Sets new virtual path provider when application starts.
    /// </summary>
    public static void AppInitialize()
    {
        HostingEnvironment.RegisterVirtualPathProvider(new DbPathProvider());
        SettingsKeyProvider.UsingVirtualPathProvider = true;
    }


    /// <summary>
    /// DbPathProvider constructor.
    /// </summary>
    public DbPathProvider()
        : base()
    {
    }


    /// <summary>
    /// Finds whether current path contains virtual directory.
    /// </summary>
    /// <returns>True or false.</returns>
    private bool IsPathVirtual()
    {
        switch (this.VirtualDirectory)
        {
            case VD_TRANSFORMATIONS:
                return true;

            case VD_WEBPARTS:
                return true;

            case VD_PAGELAYOUTS:
                return true;

            case VD_LAYOUTS:
                return true;

            case VD_CSSSTYLESHEETS:
                return true;

            default:
                return false;
        }
    }


    /// <summary>
    /// Finds whether the file, which is part of given virtual path, exists
    /// on the virtual file system or on the normal file system.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns>True or false.</returns>
    public override bool FileExists(string virtualPath)
    {
        this.VirtualPath = virtualPath;
        if (IsPathVirtual())
        {
            // Determine whether the file exists on the virtual file system.
            try
            {
                switch (this.VirtualDirectory)
                {
                    // Transformation
                    case VD_TRANSFORMATIONS:
                        return (this.Transformation != null);

                    // Layout
                    case VD_LAYOUTS:
                        return (this.Layout != null);

                    // Page layout
                    case VD_PAGELAYOUTS:
                        return (this.PageLayout != null);

                    // Stylesheet
                    case VD_CSSSTYLESHEETS:
                        return (this.Stylesheet != null);

                    // Web part
                    case VD_WEBPARTS:
                        // Do not check, loads dynamically within processing
                        return true;
                }
            }
            catch
            {
                return Previous.FileExists(virtualPath);
            }
        }

        // If not, the file is sought on the normal file system
        return Previous.FileExists(virtualPath);
    }


    /// <summary>
    /// Finds whether the directory, which is part of given virtual path, exists
    /// on the virtual file system or on the normal file system.
    /// </summary>
    /// <param name="virtualDir"></param>
    /// <returns>True or false.</returns>
    public override bool DirectoryExists(string virtualDir)
    {
        this.VirtualPath = virtualDir;
        if (IsPathVirtual())
        {
            return true;
        }
        else
        {
            return Previous.DirectoryExists(virtualDir);
        }
    }


    /// <summary>
    /// This method is used by the compilation system to obtain a VirtualFile instance to 
    /// work with a given virtual file path.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns>Virtual File.</returns>
    public override VirtualFile GetFile(string virtualPath)
    {
        this.VirtualPath = virtualPath;

        if (IsPathVirtual())
        {
            return new DbVirtualFile(virtualPath, this);
        }

        return Previous.GetFile(virtualPath);
    }


    /// <summary>
    /// This method is used by the compilation system to obtain a VirtualDirectory 
    /// instance to work with a given virtual directory.
    /// </summary>
    /// <param name="virtualDir"></param>
    /// <returns>Virtual Directory.</returns>
    public override VirtualDirectory GetDirectory(string virtualDir)
    {
        this.VirtualPath = virtualDir;
        if (IsPathVirtual())
        {
            return new DbVirtualDirectory(virtualDir, this);
        }
        else
        {
            return Previous.GetDirectory(virtualDir);
        }
    }


    /// <summary>
    /// Returns virtual directories from the virtual path
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns></returns>
    private string GetVirtDirectory(string virtualPath)
    {
        // Get local virtual path
        String checkPath = VirtualPathUtility.ToAppRelative(virtualPath);

        // Check the directory
        if (checkPath.StartsWith(TransformationInfoProvider.TransformationsDirectory, StringComparison.InvariantCultureIgnoreCase))
        {
            // Transformations
            return VD_TRANSFORMATIONS;
        }
        else if (checkPath.StartsWith(LayoutInfoProvider.LayoutsDirectory, StringComparison.InvariantCultureIgnoreCase))
        {
            // Layouts
            return VD_LAYOUTS;
        }
        else if (checkPath.StartsWith(PageTemplateInfoProvider.TemplateLayoutsDirectory, StringComparison.InvariantCultureIgnoreCase))
        {
            // Page layouts
            return VD_PAGELAYOUTS;
        }
        else if (checkPath.StartsWith(CssStylesheetInfoProvider.CSSStylesheetsDirectory, StringComparison.InvariantCultureIgnoreCase))
        {
            // CSS Stylesheets
            return VD_CSSSTYLESHEETS;
        }
        else if (checkPath.StartsWith(WebPartInfoProvider.VirtualWebPartsDirectory, StringComparison.InvariantCultureIgnoreCase))
        {
            // Web parts
            return VD_WEBPARTS;
        }
        else
        {
            return "";
        }
    }


    /// <summary>
    /// Gets string content of the file, which is part of given transformated virtual path.
    /// </summary>
    /// <param name="vPath">Virtual path.</param>
    public string GetFileContent(string virtualPath)
    {
        this.VirtualPath = virtualPath;

        string content = null;
        string editDeleteButton = "";

        switch (this.VirtualDirectory)
        {
            case VD_TRANSFORMATIONS:
                // Get transformation code
                if (this.Transformation != null)
                {
                    if (virtualPath.Contains("---showeditdelete"))
                    {
                        editDeleteButton = "<cc1:CMSEditModeButtonEditDelete runat=\"server\" id=\"btnEditDeleteAutoInsert\" Path='<%# Eval(\"NodeAliasPath\") %>' />";
                    }

                    TransformationInfo ti = this.Transformation;
                    if ((ti.TransformationCheckedOutByUserID > 0) && (ti.TransformationCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()) && (File.Exists(HttpContext.Current.Server.MapPath(ti.TransformationCheckedOutFilename))))
                    {
                        TextReader tr = new StreamReader(HttpContext.Current.Server.MapPath(ti.TransformationCheckedOutFilename));
                        content = tr.ReadToEnd();
                        tr.Close();
                    }
                    else
                    {
                        content = GetTransformationDirectives() + this.Transformation.TransformationCode + editDeleteButton;
                    }
                }
                else
                {
                    throw new Exception("[DbPathProvider]: Transformation '" + mObjectName + "' not found!");
                }
                break;

            case VD_LAYOUTS:

                // Get layout code
                if (this.Layout != null)
                {
                    LayoutInfo li = this.Layout;
                    if ((li.LayoutCheckedOutByUserID > 0) && (li.LayoutCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()) && (File.Exists(HttpContext.Current.Server.MapPath(li.LayoutCheckedOutFilename))))
                    {
                        TextReader tr = new StreamReader(HttpContext.Current.Server.MapPath(li.LayoutCheckedOutFilename));
                        content = tr.ReadToEnd();
                        tr.Close();
                    }
                    else
                    {
                        content = LayoutInfoProvider.GetLayoutDirectives() + li.LayoutCode;
                    }
                }
                else
                {
                    throw new Exception("[DbPathProvider]: Layout '" + mObjectName + "' not found!");
                }
                break;

            case VD_PAGELAYOUTS:

                // Get layout code
                if (this.PageLayout != null)
                {
                    PageTemplateInfo pti = this.PageLayout;
                    if ((pti.PageTemplateLayoutCheckedOutByUserID > 0) && (pti.PageTemplateLayoutCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()) && (File.Exists(HttpContext.Current.Server.MapPath(pti.PageTemplateLayoutCheckedOutFileName))))
                    {
                        TextReader tr = new StreamReader(HttpContext.Current.Server.MapPath(pti.PageTemplateLayoutCheckedOutFileName));
                        content = tr.ReadToEnd();
                        tr.Close();
                    }
                    else
                    {
                        content = LayoutInfoProvider.GetLayoutDirectives() + pti.PageTemplateLayout;
                    }
                }
                else
                {
                    throw new Exception("[DbPathProvider]: Page layout '" + mObjectName + "' not found!");
                }
                break;

            case VD_CSSSTYLESHEETS:
                // Get stylesheet code
                if (this.Stylesheet != null)
                {
                    CssStylesheetInfo si = this.Stylesheet;
                    if ((si.StylesheetCheckedOutByUserID > 0) && (si.StylesheetCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()) && (File.Exists(HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename))))
                    {
                        TextReader tr = new StreamReader(HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename));
                        content = tr.ReadToEnd();
                        tr.Close();
                    }
                    else
                    {
                        content = si.StylesheetText;
                    }
                }
                else
                {
                    throw new Exception("[DbPathProvider]: CssStylesheet '" + mObjectName + "' not found!");
                }
                break;

            case VD_WEBPARTS:
                // Get webpart code
                WebPartInstance webPart = this.WebPart;
                if (webPart != null)
                {
                    // Get web part
                    WebPartInfo part = WebPartInfoProvider.GetWebPartInfo(webPart.WebPartType);
                    if (part != null)
                    {
                        // Load the original web part file
                        string webPartFile = HttpContext.Current.Server.MapPath(WebPartInfoProvider.GetWebPartUrl(part));
                        TextReader reader = new StreamReader(webPartFile);
                        content = reader.ReadToEnd();
                        reader.Close();

                        PageInfo currentPage = CMSContext.CurrentPageInfo;
                        if ((TreePathUtils.IsMenuItemType(currentPage.ClassName) && (CMSContext.CurrentUser.MenuItemViewMode != ViewModeEnum.Design)) || (!TreePathUtils.IsMenuItemType(currentPage.ClassName) && (CMSContext.CurrentUser.MenuItemViewMode != ViewModeEnum.Design)))
                        {
                            // Add the additional code
                            content += "\n" + webPart.GetValue("WebPartCode");
                        }
                    }
                    else
                    {
                        throw new Exception("[DbPathProvider.GetFileContent]: Web part '" + webPart.WebPartType + "' not found!");
                    }
                }
                else
                {
                    throw new Exception("[DbPathProvider]: Web part '" + mObjectName + "' not found!");
                }
                break;

            default:
                content = "";
                break;
        }

        return content;
    }


    /// <summary>
    /// Overriden method 'GetCacheDependency()'.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <param name="virtualPathDependencies"></param>
    /// <param name="utcStart"></param>
    /// <returns>CacheDependency.</returns>
    public override CacheDependency GetCacheDependency(string virtualPath, System.Collections.IEnumerable virtualPathDependencies, DateTime utcStart)
    {
        this.VirtualPath = virtualPath;
        if (IsPathVirtual())
        {
            // Ensure the object record within cache
            string cacheName = this.VirtualDirectory + "|" + mObjectName;

            if (HttpContext.Current.Cache[cacheName] == null)
            {
                HttpContext.Current.Cache.Add(cacheName, DateTime.Now, null, DateTime.Now.AddHours(10), Cache.NoSlidingExpiration, CacheItemPriority.NotRemovable, null);
            }

            // Set file dependency
            string[] fileDependency = null;
            switch (this.VirtualDirectory)
            {
                // Layout
                case VD_LAYOUTS:
                    LayoutInfo li = this.Layout;
                    if ((li != null) && (li.LayoutCheckedOutByUserID > 0) && (li.LayoutCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()))
                    {
                        fileDependency = new string[] { HttpContext.Current.Server.MapPath(li.LayoutCheckedOutFilename) };
                    }
                    break;

                // Page layout
                case VD_PAGELAYOUTS:
                    PageTemplateInfo pti = this.PageLayout;
                    if ((pti != null) && (pti.PageTemplateLayoutCheckedOutByUserID > 0) && (pti.PageTemplateLayoutCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()))
                    {
                        fileDependency = new string[] { HttpContext.Current.Server.MapPath(pti.PageTemplateLayoutCheckedOutFileName) };
                    }
                    break;

                // Transformation
                case VD_TRANSFORMATIONS:
                    TransformationInfo ti = this.Transformation;
                    if ((ti != null) && (ti.TransformationCheckedOutByUserID > 0) && (ti.TransformationCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()))
                    {
                        fileDependency = new string[] { HttpContext.Current.Server.MapPath(ti.TransformationCheckedOutFilename) };
                    }
                    break;

                // Stylesheet
                case VD_CSSSTYLESHEETS:
                    CssStylesheetInfo si = this.Stylesheet;
                    if ((si != null) && (si.StylesheetCheckedOutByUserID > 0) && (si.StylesheetCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()))
                    {
                        fileDependency = new string[] { HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename) };
                    }
                    break;

                // Webpart
                case VD_WEBPARTS:
                    WebPartInstance part = this.WebPart;
                    if (part != null)
                    {
                        string webPartFile = HttpContext.Current.Server.MapPath(WebPartInfoProvider.GetWebPartUrl(part.WebPartType));
                        fileDependency = new string[] { webPartFile };
                    }
                    break;

            }

            if (fileDependency == null)
            {
                fileDependency = new string[] { };
            }

            return new CacheDependency(fileDependency, new string[] { cacheName }, utcStart);
        }
        else
        {
            return Previous.GetCacheDependency(virtualPath, virtualPathDependencies, utcStart);
        }
    }


    /// <summary>
    /// Overriden method 'GetFileHash'
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <param name="virtualPathDependencies"></param>
    /// <returns></returns>
    public override string GetFileHash(string virtualPath, IEnumerable virtualPathDependencies)
    {
        return base.GetFileHash(virtualPath, virtualPathDependencies);
        //return null;
    }


    /// <summary>
    /// Transformation directives are set before transformation code.
    /// </summary>
    /// <returns></returns>
    public static string GetTransformationDirectives()
    {
        // Get language code from web.config.
        string lang = CMS.GlobalHelper.DataHelper.GetNotEmpty(System.Configuration.ConfigurationManager.AppSettings["CMSProgrammingLanguage"], "C#");
        return "<%@ Control Language=\"" + lang + "\" AutoEventWireup=\"true\" Inherits=\"CMS.Controls.CMSAbstractTransformation\" %>" + "\n" + "<%@ Register TagPrefix=\"cc1\" Namespace=\"CMS.Controls\" Assembly=\"CMS.Controls\" %>" + "\n";
    }

}
