using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;

using CMS.SiteProvider;
using CMS.GlobalHelper;
using CMS.CMSHelper;

public partial class CMSPages_GetCSS : System.Web.UI.Page
{
    /// <summary>
    /// Set to false to disable the client caching
    /// </summary>
    protected bool useClientCache = true;

    protected CMSOutputCSS outputFile = null;

    protected CurrentUserInfo currentUser = null;
    protected CurrentSiteInfo currentSite = null;

    protected CssStylesheetInfo si = null;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Get the site
        currentSite = CMSContext.CurrentSite;
        if (currentSite == null)
        {
            throw new Exception("Site not running.");
        }
        currentUser = CMSContext.CurrentUser;

        // Get the stylesheetinfo
        string stylesheetName = ValidationHelper.GetString(Request.QueryString["stylesheetname"], "");
        if (stylesheetName != "")
        {
            si = CssStylesheetInfoProvider.GetCssStylesheetInfo(stylesheetName);
        }

        string useCacheItemName = CMSContext.BaseCacheKey + "|" + this.Context.Request.Url.ToString();

        // Seek the stylesheet in cache
        if ( CacheHelper.CacheImageEnabled(currentSite.SiteName) && (currentUser.ViewMode == ViewModeEnum.LiveSite) )
        {
            // Get current OutputFile object
            outputFile = (CMSOutputCSS)CacheHelper.GetItem(useCacheItemName);
        }

        // If not valid any more, drop
        if (outputFile != null)
        {
            if ((si == null) || (outputFile.VersionGUID != si.StylesheetVersionGUID))
            {
                outputFile = null;
            }
        }

        // Load new
        if (outputFile == null)
        {
            // Get stylesheet, process it and store it in cache if caching enabled.
            ProcessStylesheet();

            // Save to cache if enabled and live site mode
            if (CacheHelper.CacheImageEnabled(currentSite.SiteName) && (currentUser.ViewMode == ViewModeEnum.LiveSite))
            {
                CacheHelper.Add(useCacheItemName, outputFile, null, DateTime.Now.AddMinutes(CacheHelper.CacheImageMinutes(currentSite.SiteName)), System.Web.Caching.Cache.NoSlidingExpiration);
            }
        }

        // Send the data
        SendFile(outputFile);
    }


    /// <summary>
    /// Sends the given file within response
    /// </summary>
    /// <param name="file">File to send</param>
    protected void SendFile(CMSOutputCSS file)
    {
        // Clear response.
        Response.Cookies.Clear();
        Response.Clear();

        Response.Cache.SetRevalidation(HttpCacheRevalidation.AllCaches);

        // Send the file
        if (si != null)
        {
            if ((file != null) && (file.OutputData != null))
            {
                // Client caching - only on the live site
                if (useClientCache && (currentUser.ViewMode == ViewModeEnum.LiveSite) && (CacheHelper.CacheImageEnabled(currentSite.SiteName)))
                {
                    // Determine the last modified date and etag sent from the browser
                    string ifModifiedString = Request.Headers["If-Modified-Since"];
                    if (ifModifiedString != null)
                    {
                        // IE-browsers send something like this:
                        // If-Modified-Since: Wed, 19 Jul 2006 11:19:59 GMT; length=3350
                        DateTime ifModified;
                        if (DateTime.TryParse(ifModifiedString.Split(";".ToCharArray())[0], out ifModified))
                        {
                            // If not changed, let the browser use the cached data
                            if (file.LastModified <= ifModified.AddSeconds(1))
                            {
                                Response.StatusCode = (int)System.Net.HttpStatusCode.NotModified;
                                Response.End();
                                return;
                            }
                        }
                    }
                }

                // Prepare response
                Response.ContentType = "text/css";

                if ( useClientCache && (currentUser.ViewMode == ViewModeEnum.LiveSite) && (CacheHelper.CacheImageEnabled(currentSite.SiteName)))
                {
                    // Send last modified header to allow client caching
                    Response.Cache.SetLastModified(file.LastModified);
                    Response.Cache.SetExpires(DateTime.Now.AddHours(24));
                    Response.Cache.SetCacheability(HttpCacheability.Public);
                }

                // Add the file data
                Response.Write(file.OutputData);
            }
        }

        Response.End();
    }


    /// <summary>
    /// Processes the stylesheet.
    /// </summary>
    protected void ProcessStylesheet()
    {
        if (si != null)
        {
            string content = null;

            if ((si.StylesheetCheckedOutByUserID > 0) && (si.StylesheetCheckedOutMachineName.ToLower() == HttpContext.Current.Server.MachineName.ToLower()) && (File.Exists(HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename))))
            {
                TextReader tr = new StreamReader(HttpContext.Current.Server.MapPath(si.StylesheetCheckedOutFilename));
                content = tr.ReadToEnd();
                tr.Close();
            }
            else
            {
                content = si.StylesheetText;
            }

            outputFile = new CMSOutputCSS();
            outputFile.OutputData = content;
            outputFile.LastModified = DateTime.Now;
            outputFile.VersionGUID = si.StylesheetVersionGUID;
        }
    }
}
